/*
 * Copyright (c) 2004 Richard Longfellow.  All rights reserved.
 * Copyright (c) 2004 Chris Wareham.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package net.chriswareham.showbarcode;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;

/**
 * This abstract class is implemented by bar code encoders.
 */
public abstract class BarCode implements Printable {
    /**
     * Set the unencoded string of text.
     *
     * @param text the unencoded string of text
     */
    public abstract void setText(String text);

    /**
     * Get the unencoded string of text.
     *
     * @return the unencoded string of text
     */
    public abstract String getText();

    /**
     * Get the encoded string of text.
     *
     * @return the encoded string of text
     */
    public abstract String getLines();

    /**
     * Draw the bar code on a graphics context.
     *
     * @param graphics the context to draw the bar code on
     * @param size the dimensions to draw the bar code in
     */
    public void draw(final Graphics graphics, final Dimension size) {
        draw(graphics, 0, 0, size);
    }

    /**
     * Draw the bar code on a graphics context.
     *
     * @param graphics the context to draw the bar code on
     * @param x the x offset to draw the bar code at
     * @param y the y offset to draw the bar code at
     * @param size the dimensions to draw the bar code in
     */
    public void draw(final Graphics graphics, final int x, final int y, final Dimension size) {
        String lines = getLines();

        int length = lines.length();

        if (length == 0 || length > size.width) {
            graphics.setColor(Color.white);
            graphics.fillRect(x, y, size.width, size.height);
        } else {
            int width = size.width / length;
            int offset = x;

            for (int i = 0; i < length; ++i) {
                graphics.setColor(lines.charAt(i) == '0' ? Color.WHITE : Color.BLACK);
                graphics.fillRect(offset, y, width, size.height);
                offset += width;
            }

            graphics.fillRect(offset, 0, size.width - offset, size.height);
        }
    }

    /**
     * Save the bar code to a file as an image.
     *
     * @param filename the name of the file to save the bar code image to
     * @param size the size of the image
     * @throws IOException if an error occurs saving the bar code image
     */
    public void save(final String filename, final Dimension size) throws IOException {
        BufferedImage bufferedImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_3BYTE_BGR);

        Graphics graphics = bufferedImage.createGraphics();

        draw(graphics, size);

        ImageOutputStream imageOutputStream = ImageIO.createImageOutputStream(new File(filename + ".jpg"));

        ImageWriter imageWriter = (ImageWriter) ImageIO.getImageWritersByFormatName("jpeg").next();
        imageWriter.setOutput(imageOutputStream);
        imageWriter.write(bufferedImage);
    }

    /**
     * Print the bar code on a graphics context.
     *
     * @param graphics the context to print the bar code on
     * @param pageFormat the size and orientation of the page being drawn
     * @param pageIndex the zero based index of the page to be drawn
     * @return PAGE_EXISTS if the page is rendered successfully or NO_SUCH_PAGE if pageIndex specifies a non-existent page
     * @throws PrinterException if an error occurs printing the bar code
     */
    public int print(final Graphics graphics, final PageFormat pageFormat, final int pageIndex) throws PrinterException {
        if (pageIndex > 0) {
            return NO_SUCH_PAGE;
        }

        graphics.setClip(0, 0,
            (int) pageFormat.getImageableWidth(),
            (int) pageFormat.getImageableHeight());

        graphics.translate(
            (int) pageFormat.getImageableX(),
            (int) pageFormat.getImageableY());

        int requiredWidth = getLines().length();

        Dimension size = new Dimension(requiredWidth, requiredWidth / 3);

        draw(graphics, size);

        System.gc();

        return PAGE_EXISTS;
    }
}
