/*
 * Copyright (c) 2004 Richard Longfellow. All rights reserved.
 * Copyright (c) 2004 Chris Wareham. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package net.chriswareham.showbarcode;

import java.awt.Graphics;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.IOException;

import javax.swing.JPanel;

/**
 * This class provides a panel for displaying bar codes.
 */
public class BarCodePanel extends JPanel implements Printable {
    /**
     * The bar code encoder.
     */
    private BarCode barCode;

    /**
     * Construct a a panel for displaying bar codes.
     *
     * @param bc the bar code encoder
     */
    public BarCodePanel(final BarCode bc) {
        barCode = bc;
    }

    /**
     * Get the unencoded string of text.
     *
     * @return text the unencoded string of text
     */
    public String getText() {
        return barCode.getText();
    }

    /**
     * Set the unencoded string of text and render the encoded result.
     *
     * @param text the unencoded string of text
     */
    public void setText(final String text) {
        barCode.setText(text);
        repaint();
    }

    /**
     * Saves the bar code to a file as an image.
     *
     * @param filename the name of the file to save the bar code image to
     * @throws IOException if an error occurs saving the bar code image
     */
    public void save(final String filename) throws IOException {
        barCode.save(filename, getSize());
    }

    /**
     * Print the bar code on a graphics context.
     *
     * @param graphics the context to print the bar code on
     * @param pageFormat the size and orientation of the page being drawn
     * @param pageIndex the zero based index of the page to be drawn
     * @return PAGE_EXISTS if the page is rendered successfully or NO_SUCH_PAGE if pageIndex specifies a non-existent page
     * @throws PrinterException if an error occurs printing the bar code
     */
    @Override public int print(final Graphics graphics, final PageFormat pageFormat, final int pageIndex) throws PrinterException {
        return barCode.print(graphics, pageFormat, pageIndex);
    }

    /**
     * Render the encoded bar code on a graphics context.
     *
     * @param graphics the context to render the bar code on
     */
    @Override public void paintComponent(final Graphics graphics) {
        barCode.draw(graphics, getSize());
    }
}
