/*
 * Copyright (c) 2004 Richard Longfellow. All rights reserved.
 * Copyright (c) 2004 Chris Wareham. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package net.chriswareham.showbarcode;

/**
 * This class implements a Code 128 bar code encoder.
 */
public class Code128 extends BarCode {
    /** Encoding type A. */
    private static final int TYPE_A = 0;
    /** Encoding type B. */
    private static final int TYPE_B = 1;
    /** Encoding type C. */
    private static final int TYPE_C = 2;
    /** Encoding type A start offset. */
    private static final int TYPE_A_START_OFFSET = 103;
    /** Encoding type B start offset. */
    private static final int TYPE_B_START_OFFSET = 104;
    /** Encoding type C start offset. */
    private static final int TYPE_C_START_OFFSET = 105;
    /** Encoding stop offset. */
    private static final int STOP_OFFSET         = 106;
    /** Padding for start and end of encoded string. */
    private static final String PADDING = "00000000000";
    /** Checksum moduli. */
    private static final int CHECKSUM_MODULI = 103;
    /** Encoding matrix. */
    private static final String[][] CODE_128 = {
        { " ",      " ",      "00",     "212222"  },
        { "!",      "!",      "01",     "222122"  },
        { "\"",     "\"",     "02",     "222221"  },
        { "#",      "#",      "03",     "121223"  },
        { "$",      "$",      "04",     "121322"  },
        { "%",      "%",      "05",     "131222"  },
        { "&",      "&",      "06",     "122213"  },
        { "'",      "'",      "07",     "122312"  },
        { "(",      "(",      "08",     "132212"  },
        { ")",      ")",      "09",     "221213"  },
        { "*",      "*",      "10",     "221312"  },
        { "+",      "+",      "11",     "231212"  },
        { ",",      ",",      "12",     "112232"  },
        { "-",      "-",      "13",     "122132"  },
        { ".",      ".",      "14",     "122231"  },
        { "/",      "/",      "15",     "113222"  },
        { "0",      "0",      "16",     "123122"  },
        { "1",      "1",      "17",     "123221"  },
        { "2",      "2",      "18",     "223211"  },
        { "3",      "3",      "19",     "221132"  },
        { "4",      "4",      "20",     "221231"  },
        { "5",      "5",      "21",     "213212"  },
        { "6",      "6",      "22",     "223112"  },
        { "7",      "7",      "23",     "312131"  },
        { "8",      "8",      "24",     "311222"  },
        { "9",      "9",      "25",     "321122"  },
        { ":",      ":",      "26",     "321221"  },
        { ";",      ";",      "27",     "312212"  },
        { "<",      "<",      "28",     "322112"  },
        { "=",      "=",      "29",     "322211"  },
        { ">",      ">",      "30",     "212123"  },
        { "?",      "?",      "31",     "212321"  },
        { "@",      "@",      "32",     "232121"  },
        { "A",      "A",      "33",     "111323"  },
        { "B",      "B",      "34",     "131123"  },
        { "C",      "C",      "35",     "131321"  },
        { "D",      "D",      "36",     "112313"  },
        { "E",      "E",      "37",     "132113"  },
        { "F",      "F",      "38",     "132311"  },
        { "G",      "G",      "39",     "211313"  },
        { "H",      "H",      "40",     "231113"  },
        { "I",      "I",      "41",     "231311"  },
        { "J",      "J",      "42",     "112133"  },
        { "K",      "K",      "43",     "112331"  },
        { "L",      "L",      "44",     "132131"  },
        { "M",      "M",      "45",     "113123"  },
        { "N",      "N",      "46",     "113321"  },
        { "O",      "O",      "47",     "133121"  },
        { "P",      "P",      "48",     "313121"  },
        { "Q",      "Q",      "49",     "211331"  },
        { "R",      "R",      "50",     "231131"  },
        { "S",      "S",      "51",     "213113"  },
        { "T",      "T",      "52",     "213311"  },
        { "U",      "U",      "53",     "213131"  },
        { "V",      "V",      "54",     "311123"  },
        { "W",      "W",      "55",     "311321"  },
        { "X",      "X",      "56",     "331121"  },
        { "Y",      "Y",      "57",     "312113"  },
        { "Z",      "Z",      "58",     "312311"  },
        { "[",      "[",      "59",     "332111"  },
        { "\\",     "\\",     "60",     "314111"  },
        { "]",      "]",      "61",     "221411"  },
        { "^",      "^",      "62",     "431111"  },
        { "_",      "_",      "63",     "111224"  },
        { "NUL",    "`",      "64",     "111422"  },
        { "SOH",    "a",      "65",     "121124"  },
        { "STX",    "b",      "66",     "121421"  },
        { "ETX",    "c",      "67",     "141122"  },
        { "EOT",    "d",      "68",     "141221"  },
        { "ENQ",    "e",      "69",     "112214"  },
        { "ACK",    "f",      "70",     "112412"  },
        { "BEL",    "g",      "71",     "122114"  },
        { "BS",     "h",      "72",     "122411"  },
        { "HT",     "i",      "73",     "142112"  },
        { "LF",     "j",      "74",     "142211"  },
        { "VT",     "k",      "75",     "241211"  },
        { "FF",     "l",      "76",     "221114"  },
        { "CR",     "m",      "77",     "413111"  },
        { "SO",     "n",      "78",     "241112"  },
        { "SI",     "o",      "79",     "134111"  },
        { "DLE",    "p",      "80",     "111242"  },
        { "DC1",    "q",      "81",     "121142"  },
        { "DC2",    "r",      "82",     "121241"  },
        { "DC3",    "s",      "83",     "114212"  },
        { "DC4",    "t",      "84",     "124112"  },
        { "NAK",    "u",      "85",     "124211"  },
        { "SYN",    "v",      "86",     "411212"  },
        { "ETB",    "w",      "87",     "421112"  },
        { "CAN",    "x",      "88",     "421211"  },
        { "EM",     "y",      "89",     "212141"  },
        { "SUB",    "z",      "90",     "214121"  },
        { "ESC",    "{",      "91",     "412121"  },
        { "FS",     "|",      "92",     "111143"  },
        { "GS",     "}",      "93",     "111341"  },
        { "RS",     "~",      "94",     "131141"  },
        { "US",     "DEL",    "95",     "114113"  },
        { "FNC3",   "FNC3",   "96",     "114311"  },
        { "FNC2",   "FNC2",   "97",     "411113"  },
        { "SHIFT",  "SHIFT",  "98",     "411311"  },
        { "CODEC",  "CODEC",  "99",     "113141"  },
        { "CODEB",  "FNC4",   "CODEB",  "114131"  },
        { "FNC4",   "CODEA",  "CODEA",  "311141"  },
        { "FNC1",   "FNC1",   "FNC1",   "411131"  },
        { "StartA", "StartA", "StartA", "211412"  },
        { "StartB", "StartB", "StartB", "211214"  },
        { "StartC", "StartC", "StartC", "211232"  },
        { "Stop",   "Stop",   "Stop",   "2331112" }
    };

    /** The encoding type. */
    private int barCodeType;
    /** The unencoded string of text. */
    private String barCodeText;
    /** The encoded string of text. */
    private String barCodeLines;

    /**
     * Construct a Code 128 bar code encoder.
     */
    public Code128() {
        setText("", TYPE_B);
    }

    /**
     * Construct a Code 128 bar code encoder.
     *
     * @param text the string of text to encode
     */
    public Code128(final String text) {
        setText(text, TYPE_B);
    }

    /**
     * Construct a Code 128 bar code encoder.
     *
     * @param text the string of text to encode
     * @param type the encoding type
     */
    public Code128(final String text, final int type) {
        setText(text, type);
    }

    /**
     * {@inheritDoc}
     */
    @Override public String getText() {
        return barCodeText;
    }

    /**
     * {@inheritDoc}
     */
    @Override public void setText(final String text) {
        setText(text, TYPE_B);
    }

    /**
     * Set the unencoded string of text.
     *
     * @param text the unencoded string of text
     * @param type the encoding type
     */
    public void setText(final String text, final int type) {
        barCodeText = text;

        if (type < TYPE_A || type > TYPE_C) {
            barCodeType = TYPE_B;
        } else {
            barCodeType = type;
        }

        // create encoded string

        StringBuilder buf = new StringBuilder();

        int checkSum = 0;

        // add start padding

        buf.append(PADDING);

        // add start code

        switch (barCodeType) {
        case TYPE_A:
            buf.append(encodeChar(CODE_128[TYPE_A_START_OFFSET][3]));
            checkSum = TYPE_A_START_OFFSET;
            break;
        case TYPE_B:
            buf.append(encodeChar(CODE_128[TYPE_B_START_OFFSET][3]));
            checkSum = TYPE_B_START_OFFSET;
            break;
        case TYPE_C:
            buf.append(encodeChar(CODE_128[TYPE_C_START_OFFSET][3]));
            checkSum = TYPE_C_START_OFFSET;
            break;
        }

        // add encoded characters

        int length = text.length();

        for (int i = 0; i < length; ++i) {
            char c = barCodeText.charAt(i);

            for (int j = 0; j < 95; ++j) {
                if (CODE_128[j][barCodeType].charAt(0) == c) {
                    buf.append(encodeChar(CODE_128[j][3]));
                    checkSum += (i + 1) * j;
                    break;
                }
            }
        }

        // add checksum

        buf.append(encodeChar(CODE_128[checkSum % CHECKSUM_MODULI][3]));

        // add stop code

        buf.append(encodeChar(CODE_128[STOP_OFFSET][3]));

        // add end padding

        buf.append(PADDING);

        barCodeLines = buf.toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override public String getLines() {
        return barCodeLines;
    }

    /**
     * Encode a character.
     *
     * @param code the character to encode
     * @return the encoded character
     */
    private String encodeChar(final String code) {
        StringBuilder buf = new StringBuilder();

        int length = code.length();

        for (int i = 0; i < length; ++i) {
            int number = Integer.parseInt(code.substring(i, i + 1));

            for (int j = 0; j < number; ++j) {
                buf.append((i % 2 == 0) ? '1' : '0');
            }
        }

        return buf.toString();
    }

    /**
     * Test harness.
     *
     * @param args the command line arguments
     */
    public static void main(final String[] args) {
        Code128 bc = new Code128();
        for (int i = 0; i < args.length; ++i) {
            System.out.println("Argument " + args[i]);
            bc.setText(args[i]);
            System.out.println("Text " + bc.getText());
            System.out.println("Lines " + bc.getLines());
        }
    }
}
