/*
 * Copyright (c) 2004 Richard Longfellow. All rights reserved.
 * Copyright (c) 2004 Chris Wareham. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package net.chriswareham.showbarcode;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * This class provides a graphical interface for the ShowBarCode application.
 */
public class ShowBarCode extends JPanel {
    /**
     * The text field for entering the unencoded bar code.
     */
    private JTextField textField;
    /**
     * The panel for displaying the encoded bar code.
     */
    private BarCodePanel barCodePanel;

    /**
     * Run the ShowBarCode application.
     *
     * @param args the command line arguments
     */
    public static void main(final String[] args) {
        JFrame frame = new JFrame();
        frame.addWindowListener(new WindowAdapter() {
            @Override public void windowClosing(final WindowEvent event) {
                System.exit(0);
            }
        });
        frame.setSize(600, 480);
        frame.setTitle("Show Barcode");
        frame.getContentPane().add(new ShowBarCode(), BorderLayout.CENTER);
        frame.setVisible(true);
    }

    /**
     * Construct a new instance of the graphical interface for the ShowBarCode application.
     */
    public ShowBarCode() {
        super(new BorderLayout());

        JPanel panel = new JPanel(new GridBagLayout());
        add(panel, BorderLayout.NORTH);

        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(4, 4, 4, 4);
        c.fill = GridBagConstraints.HORIZONTAL;

        JLabel label = new JLabel("Barcode:");
        panel.add(label, c);

        c.weightx = 1.0;
        textField = new JTextField();
        textField.setDocument(new TextLimitDocument());
        textField.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent event) {
                draw();
            }
        });
        panel.add(textField, c);

        c.weightx = 0.0;
        JButton button = new JButton("Draw");
        button.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent event) {
                draw();
            }
        });
        panel.add(button, c);

        barCodePanel = new BarCodePanel(new Code128());
        barCodePanel.setBorder(new EtchedBorder());
        add(barCodePanel, BorderLayout.CENTER);

        panel = new JPanel();
        add(panel, BorderLayout.SOUTH);

        button = new JButton("Save");
        button.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent event) {
                save();
            }
        });
        panel.add(button);

        button = new JButton("Print");
        button.addActionListener(new ActionListener() {
            @Override public void actionPerformed(final ActionEvent event) {
                print();
            }
        });
        panel.add(button);
    }

    /**
     * Draw the currently entered text as a barcode.
     */
    private void draw() {
        barCodePanel.setText(textField.getText().trim());
    }

    /**
     * Save the currently drawn barcode.
     */
    private void save() {
        try {
            barCodePanel.save(textField.getText().trim());
        } catch (IOException exception) {
            JOptionPane.showMessageDialog(this, "Save failed : " + exception.getMessage(), "Save Failed", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Print the currently drawn barcode.
     */
    private void print() {
        PrinterJob printerJob = PrinterJob.getPrinterJob();
        printerJob.setPrintable(barCodePanel);
        printerJob.setJobName(barCodePanel.getText());
        if (printerJob.printDialog()) {
            try {
                printerJob.print();
            } catch (PrinterException exception) {
                JOptionPane.showMessageDialog(this, "Print failed : " + exception.getMessage(), "Print Failed", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * This class provides a document that limits the length of text.
     */
    private static class TextLimitDocument extends PlainDocument {
        /**
         * {@inheritDoc}
         */
        @Override public void insertString(final int offs, final String str, final AttributeSet a) throws BadLocationException {
            if ((getLength() + str.length()) <= 30) {
                super.insertString(offs, str, a);
            }
        }
    }
}
