/*
 * Copyright (c) 2004 Richard Longfellow. All rights reserved.
 * Copyright (c) 2004 Chris Wareham. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package net.chriswareham.showbarcode;

/**
 * This class implements a Two Of Five Interleave bar code encoder.
 */
public class TwoOfFiveInterleave extends BarCode {
    /**
     * The padding for the start of the encoded string.
     */
    private static final String START_PADDING = "00001010";
    /**
     * The padding for the end of the encoded string.
     */
    private static final String END_PADDING = "11010000";
    /**
     * The thin bar.
     */
    private static final short THIN = 0;
    /**
     * The wide bar.
     */
    private static final short WIDE = 1;
    /**
     * The encoding matrix.
     */
    private static final short[][] TWO_OF_FIVE = {
        { THIN, THIN, WIDE, WIDE, THIN }, // 0
        { WIDE, THIN, THIN, THIN, WIDE }, // 1
        { THIN, WIDE, THIN, THIN, WIDE }, // 2
        { THIN, WIDE, THIN, THIN, WIDE }, // 3
        { WIDE, WIDE, THIN, THIN, THIN }, // 4
        { WIDE, THIN, WIDE, THIN, THIN }, // 5
        { THIN, WIDE, WIDE, THIN, THIN }, // 6
        { THIN, THIN, THIN, WIDE, THIN }, // 7
        { WIDE, THIN, THIN, WIDE, THIN }, // 8
        { THIN, WIDE, THIN, WIDE, THIN }  // 9
    };

    /**
     * The unencoded string of text.
     */
    private String barCodeText;
    /**
     * The encoded string of text.
     */
    private String barCodeLines;

    /**
     * Construct a Two Of Five Interleave bar code encoder.
     */
    public TwoOfFiveInterleave() {
        this("");
    }

    /**
     * Construct a Two Of Five Interleave bar code encoder.
     *
     * @param text the unencoded string of text
     */
    public TwoOfFiveInterleave(final String text) {
        setText(text);
    }

    /**
     * {@inheritDoc}
     */
    @Override public String getText() {
        return barCodeText;
    }

    /**
     * {@inheritDoc}
     */
    @Override public void setText(final String text) {
        // validate string

        int length = text.length();

        if (!text.matches("\\d+")) {
            length = 0;
            barCodeText = "";
        } else if (length % 2 != 0) {
            length++;
            barCodeText = "0" + text;
        } else {
            barCodeText = text;
        }

        // create encoded string

        StringBuilder buf = new StringBuilder(length * 10 + 16);

        // add start padding

        buf.append(START_PADDING);

        // interleave

        for (int i = 0; i < length; i += 2) {
            int first = Integer.parseInt(barCodeText.substring(i, i + 1));
            int second = Integer.parseInt(barCodeText.substring(i + 1, i + 2));

            // get actual widths

            short[] firstBars = TWO_OF_FIVE[first];
            short[] secondBars = TWO_OF_FIVE[second];

            // interleave and change widths to bar colors

            for (int j = 0; j < 5; ++j) {
                buf.append(firstBars[j] == THIN ? "1" : "11");
                buf.append(secondBars[j] == THIN ? "0" : "00");
            }
        }

        // add end padding

        buf.append(END_PADDING);

        barCodeLines = buf.toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override public String getLines() {
        return barCodeLines;
    }

    /**
     * Test harness.
     *
     * @param args the command line arguments
     */
    public static void main(final String[] args) {
        TwoOfFiveInterleave bc = new TwoOfFiveInterleave();
        for (int i = 0; i < args.length; ++i) {
            System.out.println("Argument " + args[i]);
            bc.setText(args[i]);
            System.out.println("Text " + bc.getText());
            System.out.println("Lines " + bc.getLines());
        }
    }
}
